#include "color.hh"
#include "mtrand.h"

Color::Color() : r(0.0), g(0.0), b(0.0), a(1.0) {}

Color::Color(double r, double g, double b) : r(r), g(g), b(b), a(1.0) {}

Color::Color(double r, double g, double b, double a) : r(r), g(g), b(b), a(a) {}

Color::Color(Gdk::Color baseColor) : r(baseColor.get_red_p()), g(baseColor.get_green_p()), b(baseColor.get_blue_p()) {}

Color Color::randomColor() {
	MTRand drand;
	return Color(drand(), drand(), drand());
}

Color Color::fadeTo(Color color, double fadeSpeed)
{
	double deltaR = (color.getRed() - r) * fadeSpeed;
	double deltaG = (color.getGreen() - g) * fadeSpeed;
	double deltaB = (color.getBlue() - b) * fadeSpeed;
	double deltaA = (color.getAlpha() - a) * fadeSpeed;
	return Color(r + deltaR, g + deltaG, b + deltaB, a + deltaA);
}

double Color::getRed() {
	return this->r;
}

double Color::getGreen() {
	return this->g;
}

double Color::getBlue() {
	return this->b;
}

double Color::getAlpha() {
	return this->a;
}

void Color::copyToArray(double rgba[4])
{
	rgba[0] = this->r;
	rgba[1] = this->g;
	rgba[2] = this->b;
	rgba[3] = this->a;
}

void Color::putColorOnWidget(Gtk::ColorButton *widget)
{
	Gdk::Color *color = new Gdk::Color();
	Gdk::Color &refColor = *color;
	widget->get_default_colormap()->alloc_color(refColor);
	refColor.set_rgb_p(this->getRed(), this->getGreen(), this->getBlue());
	widget->set_color(refColor);
	widget->get_default_colormap()->free_color(refColor);
	delete color;
}