#include "colorsWindow.hh"
#include "configuration.hh"
#include "pathCheckbox.hh"
#include "pathsButton.hh"
#include "pathsWindow.hh"

ColorsWindow::ColorsWindow(Gtk::Window::BaseObjectType *object, Glib::RefPtr<Gnome::Glade::Xml> &refGlade) : Gtk::Window(object)
{
	PathsButton *pathsButton = 0;
	PathsWindow *pathsWindow = 0;
	Gtk::Button *closeButton = 0;
	drawPaths = 0;
	moleculeColor = 0;
	collidedMoleculeColor = 0;

	refGlade->get_widget("buttonOKColors", closeButton);
	refGlade->get_widget("color", moleculeColor);
	refGlade->get_widget("collidedColor", collidedMoleculeColor);
	refGlade->get_widget("colorChangeSpeed", colorChangeSpeed);
	// get these two below just to load our classes
	refGlade->get_widget_derived("buttonPathsWindow", pathsButton);
	refGlade->get_widget_derived("pathsWindow", pathsWindow);
	refGlade->get_widget_derived("drawPaths", drawPaths);

	closeButton->signal_clicked().connect(sigc::mem_fun(*this, &ColorsWindow::close));
	moleculeColor->signal_color_set().connect(sigc::mem_fun(*this, &ColorsWindow::updateMoleculeColor));
	collidedMoleculeColor->signal_color_set().connect(sigc::mem_fun(*this, &ColorsWindow::updateCollidedMoleculeColor));
	colorChangeSpeed->signal_value_changed().connect(sigc::mem_fun(*this, &ColorsWindow::updateColorChangeSpeed));

	Configuration *conf = Configuration::getInstance();
	conf->addConfigurationWidget(MOLECULES_COLOR, this);
	conf->addConfigurationWidget(COLLIDED_MOLECULES_COLOR, this);
	conf->addConfigurationWidget(COLOR_CHANGE_SPEED, this);
}

void ColorsWindow::updateMoleculeColor()
{
	Configuration::getInstance()->setMoleculesColor(Color(moleculeColor->get_color()));
}

void ColorsWindow::updateCollidedMoleculeColor()
{
	Configuration::getInstance()->setCollidedMoleculesColor(Color(collidedMoleculeColor->get_color()));
}

void ColorsWindow::updateColorChangeSpeed()
{
	// divided by 10 because of limitations in Glade
	Configuration::getInstance()->setColorChangeSpeed(colorChangeSpeed->get_value() / 10.0);
}

void ColorsWindow::updateDrawPaths()
{
	Configuration::getInstance()->toggleDrawPaths();
}

void ColorsWindow::updateState()
{
	Configuration *conf = Configuration::getInstance();
	conf->getMoleculesColor().putColorOnWidget(this->moleculeColor);
	conf->getCollidedMoleculesColor().putColorOnWidget(this->collidedMoleculeColor);
	this->colorChangeSpeed->set_value(conf->getColorChangeSpeed() * 10.0);
}

void ColorsWindow::close()
{
	this->hide();
}
