#include "configuration.hh"
#include "parameterWidget.hh"

Configuration *Configuration::instance = 0;

Configuration::Configuration()
{
	numberOfMolecules = 100;
	radius = 0.53;
	environmentScale = 1000.0;
	environmentType = CUBE;
	temperature = 273.15;
	duration = 60;
	pathsColors = std::vector<Color>();
	moleculesColor = Color(1.0, 1.0, 0.0);
	collidedMoleculesColor = Color(1.0, 0.0, 0.0);
	colorChangeSpeed = 0.01;
	showFPS = true;
	showCPS = true;
	showDiffusion = true;
	drawPaths = true;
	widgets = std::multimap<Parameter, ParameterWidget *>();
	synchronizing = false;
}

Configuration *Configuration::getInstance()
{
	if (!instance)
	{
		instance = new Configuration();
	}
	return instance;
}

int Configuration::getNumberOfMolecules()
{
	return this->numberOfMolecules;
}

void Configuration::setNumberOfMolecules(int n)
{
	this->numberOfMolecules = n;
	this->updateConfigurationWidgets(NUMBER_OF_MOLECULES);
}

double Configuration::getRadiusOfMolecules()
{
	return this->radius;
}

void Configuration::setRadiusOfMolecules(double r)
{
	this->radius = r;
	this->updateConfigurationWidgets(RADIUS_OF_MOLECULES);
}

double Configuration::getEnvironmentScale()
{
	return this->environmentScale;
}

void Configuration::setEnvironmentScale(double s)
{
	this->environmentScale = s;
	this->updateConfigurationWidgets(ENVIRONMENT_SCALE);
}

EnvironmentType Configuration::getEnvironmentType()
{
	return this->environmentType;
}

void Configuration::setEnvironmentType(EnvironmentType t)
{
	this->environmentType = t;
	this->updateConfigurationWidgets(ENVIRONMENT_TYPE);
}

double Configuration::getTemperature()
{
	return this->temperature;
}

void Configuration::setTemperature(double t)
{
	this->temperature = t;
	this->updateConfigurationWidgets(TEMPERATURE);
}

int Configuration::getDurationOfSimulation()
{
	return this->duration;
}

void Configuration::setDurationOfSimulation(int ns)
{
	this->duration = ns;
	this->updateConfigurationWidgets(DURATION_OF_SIMULATION);
}

Color Configuration::getPathColor(int i)
{
	if (this->pathsColors.size() <= i)
	{
		int j = this->pathsColors.size() - 1;
		while (j <= i)
		{
			this->pathsColors.push_back(Color::randomColor());
			j++;
		}
	}
	return this->pathsColors[i];
}

void Configuration::setPathColor(int i, Color c)
{
	this->pathsColors[i] = c;
}

Color Configuration::getMoleculesColor()
{
	return this->moleculesColor;
}

void Configuration::setMoleculesColor(Color c)
{
	this->moleculesColor = c;
	this->updateConfigurationWidgets(MOLECULES_COLOR);
}

Color Configuration::getCollidedMoleculesColor()
{
	return this->collidedMoleculesColor;
}

void Configuration::setCollidedMoleculesColor(Color c)
{
	this->collidedMoleculesColor = c;
	this->updateConfigurationWidgets(COLLIDED_MOLECULES_COLOR);
}

double Configuration::getColorChangeSpeed()
{
	return this->colorChangeSpeed;
}

void Configuration::setColorChangeSpeed(double v)
{
	this->colorChangeSpeed = v;
	this->updateConfigurationWidgets(COLOR_CHANGE_SPEED);
}

bool Configuration::shouldShowFPS()
{
	return this->showFPS;
}

void Configuration::toggleShowFPS()
{
	this->showFPS = !this->showFPS;
	this->updateConfigurationWidgets(SHOW_FPS);
}

bool Configuration::shouldShowCPS()
{
	return this->showCPS;
}

void Configuration::toggleShowCPS()
{
	this->showCPS = !this->showCPS;
	this->updateConfigurationWidgets(SHOW_CPS);
}

bool Configuration::shouldShowDiffusion()
{
	return this->showDiffusion;
}

void Configuration::toggleShowDiffusion()
{
	this->showDiffusion = !this->showDiffusion;
	this->updateConfigurationWidgets(SHOW_DIFFUSION);
}

bool Configuration::shouldDrawPaths()
{
	return this->drawPaths;
}

void Configuration::toggleDrawPaths()
{
	this->drawPaths = !this->drawPaths;
	this->updateConfigurationWidgets(DRAW_PATHS);
}

void Configuration::addConfigurationWidget(Parameter p, ParameterWidget *widget)
{
	this->widgets.insert(std::pair<Parameter, ParameterWidget *>(p, widget));
}

void Configuration::updateConfigurationWidgets(Parameter p)
{
	this->synchronizing = true;
	std::pair<std::multimap<Parameter, ParameterWidget *>::iterator, std::multimap<Parameter, ParameterWidget *>::iterator>
		widgets = this->widgets.equal_range(p);
	std::multimap<Parameter, ParameterWidget *>::iterator it;
	for (it = widgets.first; it != widgets.second; it++)
		it->second->updateState();
	this->synchronizing = false;
}

bool Configuration::isSynchronizing()
{
	return this->synchronizing;
}