#ifndef CONFIGURATION_HH
#define CONFIGURATION_HH

#include <map>
#include <vector>
#include "color.hh"
#include "environmentFactory.hh"

class ParameterWidget;

typedef enum parameter
{
	NUMBER_OF_MOLECULES,
	RADIUS_OF_MOLECULES,
	ENVIRONMENT_SCALE,
	ENVIRONMENT_TYPE,
	TEMPERATURE,
	DURATION_OF_SIMULATION,
	PATH_COLOR,
	MOLECULES_COLOR,
	COLLIDED_MOLECULES_COLOR,
	COLOR_CHANGE_SPEED,
	SHOW_FPS,
	SHOW_CPS,
	SHOW_DIFFUSION,
	DRAW_PATHS
} Parameter;

class Configuration
{
public:
	static Configuration *getInstance();

	int getNumberOfMolecules();
	void setNumberOfMolecules(int n);
	double getRadiusOfMolecules();
	void setRadiusOfMolecules(double r);
	double getEnvironmentScale();
	void setEnvironmentScale(double s);
	EnvironmentType getEnvironmentType();
	void setEnvironmentType(EnvironmentType t);
	double getTemperature();
	void setTemperature(double t);
	int getDurationOfSimulation();
	void setDurationOfSimulation(int ns);
	Color getPathColor(int i);
	void setPathColor(int i, Color c);
	Color getMoleculesColor();
	void setMoleculesColor(Color c);
	Color getCollidedMoleculesColor();
	void setCollidedMoleculesColor(Color c);
	double getColorChangeSpeed();
	void setColorChangeSpeed(double v);
	bool shouldShowFPS();
	void toggleShowFPS();
	bool shouldShowCPS();
	void toggleShowCPS();
	bool shouldShowDiffusion();
	void toggleShowDiffusion();
	bool shouldDrawPaths();
	void toggleDrawPaths();
	void addConfigurationWidget(Parameter p, ParameterWidget *widget);
	void updateConfigurationWidgets(Parameter p);
	bool isSynchronizing();
private:
	static Configuration *instance;

	// Simulation parameters
	int numberOfMolecules;
	double radius;
	double environmentScale;
	EnvironmentType environmentType;
	double temperature;
	int duration;
	// Visualization parameters
	std::vector<Color> pathsColors;
	Color moleculesColor;
	Color collidedMoleculesColor;
	double colorChangeSpeed;
	bool showFPS;
	bool showCPS; // collisions per second
	bool showDiffusion;
	bool drawPaths;

	// Widgets that need to be updated when configuration is changed
	std::multimap<Parameter, ParameterWidget *> widgets;
	// Lock of updateConfigurationWidgets
	bool synchronizing;

	Configuration();
};

#endif