#include <ode/ode.h>
#include <vtkActor.h>
#include <vtkCubeSource.h>
#include <vtkPolyDataMapper.h>
#include <vtkProperty.h>
#include "cubeEnvironment.hh"
#include "molecule.hh"
#include "type.hh"
#include "mtrand.h"

CubeEnvironment::CubeEnvironment(double size, double temperature) : size(size), Environment(temperature)
{
	src = vtkCubeSource::New();
	src->SetXLength(size);
	src->SetYLength(size);
	src->SetZLength(size);

	map = vtkPolyDataMapper::New();
	map->SetInput(src->GetOutput());

	graphicRepresentation->SetMapper(map);

	this->createGeometry();
}

CubeEnvironment::~CubeEnvironment()
{
	map->Delete();
	src->Delete();
	delete planeDown;
	delete planeUp;
	delete planeLeft;
	delete planeRight;
	delete planeFront;
	delete planeBack;
}

Vector CubeEnvironment::generateRandomValidPosition(Molecule *m)
{
	MTRand drand;
	double realSize = size / 2.0 - m->getType().getRadius();
	Vector tempPosition;
	do {
		tempPosition = Vector(realSize * (-1.0 + 2.0 * drand()), realSize * (-1.0 + 2.0 * drand()), realSize * (-1.0 + 2.0 * drand()));
	} while (hasMoleculeNear(tempPosition, m->getType().getRadius()));
	return tempPosition;
}

double CubeEnvironment::biggestFreePathInside() {
	return sqrt(3.0 * size);
}

void CubeEnvironment::createGeometry()
{
	planeDown = new dPlane(space->id(), 0.0, 1.0, 0.0, -size / 2.0);
	planeUp = new dPlane(space->id(), 0.0, -1.0, 0.0, -size / 2.0); 
	planeLeft = new dPlane(space->id(), 1.0, 0.0, 0.0, -size / 2.0);
	planeRight = new dPlane(space->id(), -1.0, 0.0, 0.0, -size / 2.0);
	planeFront = new dPlane(space->id(), 0.0, 0.0, 1.0, -size / 2.0);
	planeBack = new dPlane(space->id(), 0.0, 0.0, -1.0, -size / 2.0);
}