#ifndef ENVIRONMENT_H
#define ENVIRONMENT_H

#include <map>
#include <ode/ode.h>
#include "vector.hh"

#define BOLTZMANN 1.3806504e-23 // Boltzmann's constant in J/K
#define DELTA 0.1 // precision of integration for speed generation

class Molecule;
class vtkActor;

class Environment
{
public:
	Environment(double temperature);
	virtual ~Environment();
	void updateMolecules();
	bool addMolecule(Molecule *m);
	int getNumberOfMolecules();
	vtkActor *getGraphicRepresentation();
	double getTemperature();
	void setTemperature(double temp);
	dWorld *getPhysics();
	dSpace *getSpace();
	Vector generateRandomSpeedFor(Molecule *m); // in /ns
	double averageSpeedOfMolecules();
	double kineticEnergy();
	bool hasMoleculeNear(Vector position, double radius);
	Molecule *getMoleculeByBodyID(dBodyID id);
	std::map<dBodyID, Molecule *> getMolecules();
	virtual Vector generateRandomValidPosition(Molecule *m) = 0;
	virtual double biggestFreePathInside() = 0;
protected:
	dSpace *space;
	vtkActor *graphicRepresentation;
private:
	// Stores molecules by their body ID, so we can recover
	// the object quickly from its body ID, which is
	// provided by the simulation collider
	std::map<dBodyID, Molecule *> molecules;
	double temperature;
	dWorld *physics;

	bool hasMolecule(Molecule *m);
};

#endif
