#include "environmentTest.hh"
#include <cmath>
#include "cubeEnvironment.hh"

#define T 273.15 // temperature for the test environment

CPPUNIT_TEST_SUITE_REGISTRATION(EnvironmentTest);

void EnvironmentTest::setUp()
{
	env = new CubeEnvironment(100.0, T);
	Type t(1.0);
	m = new Molecule(t);
	m->addToEnvironment(env);
	m->setPosition(Vector(0.0, 0.0, 0.0));
}

void EnvironmentTest::tearDown()
{
	delete m;
	delete env;
}

void EnvironmentTest::testInsertMolecule()
{
	// the molecule is inserted when created, so it cannot be inserted again
	CPPUNIT_ASSERT(false == env->addMolecule(m));
	CPPUNIT_ASSERT(1 == env->getNumberOfMolecules());
}

void EnvironmentTest::testHasMoleculeNear()
{
	CPPUNIT_ASSERT(false == env->hasMoleculeNear(Vector(10.0, 0.0, 0.0), 1.0));
	CPPUNIT_ASSERT(true == env->hasMoleculeNear(Vector(10.0, 0.0, 0.0), 9.0));
	CPPUNIT_ASSERT(true == env->hasMoleculeNear(Vector(0.0, 0.0, 0.0), 1.0));
	CPPUNIT_ASSERT(true == env->hasMoleculeNear(Vector(10.0, 0.0, 0.0), 15.0));
}
