#include "mainWindow.hh"
#include "colorsButton.hh"
#include "colorsMenu.hh"
#include "colorsWindow.hh"
#include "cpsMenu.hh"
#include "cubeEnvironmentMenu.hh"
#include "cylinderEnvironmentMenu.hh"
#include "diffusionMenu.hh"
#include "durationSlider.hh"
#include "environmentCombo.hh"
#include "environmentSizeSpin.hh"
#include "exitButton.hh"
#include "fpsMenu.hh"
#include "menuQuit.hh"
#include "numberSpin.hh"
#include "pathColorsMenu.hh"
#include "pathsMenu.hh"
#include "pathsWindow.hh"
#include "pauseButton.hh"
#include "radiusSpin.hh"
#include "simulation.hh"
#include "simulationVisualization.hh"
#include "startButton.hh"
#include "temperatureSpin.hh"

#include <iostream>

MainWindow::MainWindow(BaseObjectType *object, const Glib::RefPtr<Gnome::Glade::Xml> &refGlade)
: Gtk::Window(object)
{
	this->refGlade = refGlade;
	this->renArea = 0;

	this->loadWidgets();

	this->signal_check_resize().connect(sigc::mem_fun(*this, &MainWindow::checkResize));
	this->maximize();

	Simulation::getInstance()->setVisualization(new SimulationVisualization(this));
	Simulation::getInstance()->setFinishCallback(sigc::mem_fun(*this->start, &StartButton::on_buttonStop_clicked));
}

void MainWindow::addRenderingArea(Gtk::Vtk::RenderingArea *renArea)
{
	this->vtkBlankArea->add(*renArea);
	this->renArea = renArea;
}

void MainWindow::loadWidgets()
{
	ColorsButton *colorsButton = 0;
	ColorsMenu *colorsMenu = 0;
	CPSMenu *cpsMenu = 0;
	CubeEnvironmentMenu *cubeEnvironmentMenu = 0;
	CylinderEnvironmentMenu *cylinderEnvironmentMenu = 0;
	DiffusionMenu *diffusionMenu = 0;
	DurationSlider *durationSlider = 0;
	EnvironmentCombo *envCombo = 0;
	EnvironmentSizeSpin *envSize = 0;
	ExitButton *exitButton = 0;
	FPSMenu *fpsMenu = 0;
	MenuQuit *menuQuit = 0;
	NumberSpin *number = 0;
	PathColorsMenu *pathColorsMenu = 0;
	PathsMenu *pathsMenu = 0;
	RadiusSpin *radius = 0;
	TemperatureSpin *temperature = 0;
	refGlade->get_widget_derived("buttonStart", start);
	refGlade->get_widget_derived("togglePause", pause);
	refGlade->get_widget("vtkBlankArea", vtkBlankArea);
	refGlade->get_widget("menu", menu);
	// gets these widgets just to load the derived classes
	refGlade->get_widget_derived("buttonColorsWindow", colorsButton);
	refGlade->get_widget_derived("colorsMenu", colorsMenu);
	refGlade->get_widget_derived("CPSMenu", cpsMenu);
	refGlade->get_widget_derived("cubeEnvironmentMenu", cubeEnvironmentMenu);
	refGlade->get_widget_derived("cylinderEnvironmentMenu", cylinderEnvironmentMenu);
	refGlade->get_widget_derived("diffusionMenu", diffusionMenu);
	refGlade->get_widget_derived("time", durationSlider);
	refGlade->get_widget_derived("environment", envCombo);
	refGlade->get_widget_derived("environmentSize", envSize);
	refGlade->get_widget_derived("buttonExit", exitButton);
	refGlade->get_widget_derived("FPSMenu", fpsMenu);
	refGlade->get_widget_derived("menuQuit", menuQuit);
	refGlade->get_widget_derived("number", number);
	refGlade->get_widget_derived("pathColorsMenu", pathColorsMenu);
	refGlade->get_widget_derived("pathsMenu", pathsMenu);
	refGlade->get_widget_derived("radius", radius);
	refGlade->get_widget_derived("temperature", temperature);
}

void MainWindow::checkResize()
{
	if (this->renArea)
		this->renArea->set_size_request(this->get_width() - SIDEBAR_WIDTH, this->get_height() - menu->get_height());
}

void MainWindow::setSidebarSensitive(bool sensitive)
{
	Gtk::Layout *sidebar;
	refGlade->get_widget("sidebar", sidebar);
	Glib::ListHandle<Gtk::Widget *> widgets = sidebar->get_children();
	Glib::ListHandle<Gtk::Widget *>::iterator it = widgets.begin();
	pause->set_sensitive(!sensitive);
	while (it != widgets.end())
	{
		// these buttons have own rules
		if (*it != start && *it != pause)
		{
			(*it)->set_sensitive(sensitive);
		}
		it++;
	}
}
void MainWindow::runSimulation()
{
	//std::cerr << "Initial kinetic energy: " << env->kineticEnergy() << std::endl;
	setSidebarSensitive(false);	
	Simulation::getInstance()->start();
}

void MainWindow::stopSimulation()
{
	//std::cerr << "Final kinetic energy: " << env->kineticEnergy() << std::endl;
	pause->set_active(false);
	setSidebarSensitive(true);
	Simulation::getInstance()->stop();
}
