#ifndef MOLECULE_H
#define MOLECULE_H

#include <ode/ode.h>
#include "color.hh"
#include "type.hh"
#include "vector.hh"

class Environment;
class Path;
class vtkActor;
class vtkPolyDataMapper;
class vtkSphereSource;

class Molecule {
public:
	Molecule(Type t);
	~Molecule();
	bool addToEnvironment(Environment *env);
	// adds this molecule to the ODE environment
	void addPhysics(dBody *body);
	void updatePhysics();
	// tells the molecule that it collided with another molecule
	void collided();

	// getters and setters
	// position in  (10^-10 m)
	Vector getPosition();
	void setPosition(Vector p);
	Vector getInitialPosition();
	// speed in /s
	Vector getSpeed();
	void setSpeed(Vector v);
	Color getColor();
	void setColor(Color c);
	void setCollidedColor(Color c, double fadeSpeed);
	void drawPath(bool shouldDraw);
	Type getType();
	vtkActor *getGraphicRepresentation();
	Path *getPath();
private:
	// speed of color fade to base color after a collision
	double colorFadeSpeed;
	bool shouldDrawPath;
	Color baseColor, collidedColor;
	dBody *body;
	Path *path;
	Type type;
	Vector lastSpeed;
	// stored as a reference so we can check whether it was filled or not
	Vector *initialPosition;
	vtkActor *graphicRepresentation;
	vtkPolyDataMapper *map;
	vtkSphereSource *src;
};

#endif
