#include "onScreenDiffusion.hh"
#include <map>
#include <vtkTextActor.h>
#include "environment.hh"
#include "molecule.hh"

#include <iostream>

OnScreenDiffusion::OnScreenDiffusion(Environment *env)
{
	this->env = env;
	this->maxElapsedTime = env->biggestFreePathInside() / env->averageSpeedOfMolecules();
}

void OnScreenDiffusion::update(double interval)
{
	char text[50];
	//sprintf(text, "Diffusion coeficient: %lf Å²/ns", this->calculateDiffusion(interval));
	sprintf(text, "Diffusion coeficient: %lf mm²/s", this->calculateDiffusion(interval));
	this->getText()->SetInput(text);
}

double OnScreenDiffusion::calculateDiffusion(double interval)
{
	std::map<dBodyID,Molecule*> molecules = env->getMolecules();
	std::map<dBodyID,Molecule*>::iterator it;
	double squaredDisplacement = 0.0;
	for (it = molecules.begin(); it != molecules.end(); it++)
	{
		Vector displacement = it->second->getPosition() - it->second->getInitialPosition();
		squaredDisplacement += displacement.dot(displacement);
	}
	squaredDisplacement /= molecules.size();
	// multiply by 1.0e-5 to convert squared Angstrons per nanosecond to squared milimeters per second
	return squaredDisplacement / (6.0 * this->maxElapsedTime) * 1.0e-5;
}
