#include "path.hh"
#include <vtkActor.h>
#include <vtkCellArray.h>
#include <vtkDataSetMapper.h>
#include <vtkLine.h>
#include <vtkPoints.h>
#include <vtkPolyData.h>
#include <vtkProperty.h>

Path::Path()
{
	points = vtkPoints::New();
	lines = vtkCellArray::New();
	data = vtkPolyData::New();
	mapper = vtkDataSetMapper::New();
	graphics = vtkActor::New();
	data->SetPoints(points);
	data->SetLines(lines);
	mapper->SetInput(data);
	graphics->SetMapper(mapper);
	lastPointID = -1;
	beforeLastPointID = -1;
}

Path::~Path()
{
	graphics->Delete();
	mapper->Delete();
	data->Delete();
	lines->Delete();
	points->Delete();
}

void Path::addPoint(Vector p)
{
	beforeLastPointID = lastPointID;
	lastPointID = points->InsertNextPoint(p.getX(), p.getY(), p.getZ());
	if (beforeLastPointID != -1)
	{
		vtkLine *nextLine = vtkLine::New();
		nextLine->GetPointIds()->SetId(0, beforeLastPointID);
		nextLine->GetPointIds()->SetId(1, lastPointID);
		lines->InsertNextCell(nextLine);
	}
	points->Modified(); // tells VTK to reproccess the points
}

void Path::updateEndPoint(Vector p)
{
	points->SetPoint(lastPointID, p.getX(), p.getY(), p.getZ());
	points->Modified(); // tells VTK to reproccess the points
}

Color Path::getColor()
{
	double rgb[3];
	graphics->GetProperty()->GetColor(rgb);
	return Color(rgb[0], rgb[1], rgb[2]);
}

void Path::setColor(Color c)
{
	graphics->GetProperty()->SetColor(c.getRed(), c.getGreen(), c.getBlue());
}

vtkActor *Path::getGraphics()
{
	return this->graphics;
}