#ifndef SIMULATION_HH
#define SIMULATION_HH

#include <ctime>
#include <gtkmm.h>
#include <ode/ode.h>
#include "vtkmm.h"

#define FRAME_DURATION 16 // time between screen updates
#define MAX_CONTACTS 10
#define STEP 0.001

class CollisionWatcher;
class CollisionWatchers;
class Environment;
class MainWindow;
class SimulationVisualization;

class Simulation
{
public:
	static Simulation *getInstance();
	bool refresh();
	void start();
	void stop();
	void pause();
	void unpause();
	void treatCollisions(dGeomID id1, dGeomID id2);
	void setVisualization(SimulationVisualization *view);
	SimulationVisualization *getVisualization();
	void setFinishCallback(sigc::slot<void> atEnd);
	CollisionWatchers *getCollisionWatchers();
private:
	static Simulation *instance;
	clock_t currentClock, lastClock;
	dJointGroup *contactJoints;
	Environment *env;
	int elapsed, maxDuration;
	sigc::connection signal;
	sigc::slot<void> atEnd;
	SimulationVisualization *view;
	CollisionWatchers *collisionWatchers;

	Simulation();
	void addMoleculesToEnvironment();
};

#endif