#include "simulationVisualization.hh"
#include <vtkCamera.h>
#include <vtkLightKit.h>
#include <vtkProp.h>
#include <vtkRenderer.h>
#include <vtkTextActor.h>
#include <vtkWindow.h>
#include "configuration.hh"
#include "mainWindow.hh"
#include "onScreenCPS.hh"
#include "onScreenFPS.hh"
#include "onScreenStatistic.hh"

#include <iostream>
SimulationVisualization::SimulationVisualization(MainWindow *parentWindow)
{
	this->renArea = new Gtk::Vtk::RenderingArea();
	vtkRenderer *renderer = this->renArea->get_vtk_renderer();
	vtkLightKit::New()->AddLightsToRenderer(renderer);
	renderer->SetBackground(0.0, 0.0, 0.0);
	parentWindow->addRenderingArea(this->renArea);
	this->renArea->show();
	this->onScreenStats = std::list<OnScreenStatistic *>();
	this->nextOnScreenStatHeight = BASE_ON_SCREEN_STAT_HEIGHT;
}

SimulationVisualization::~SimulationVisualization()
{
	if (this->renArea)
		delete this->renArea;
	this->resetOnScreenStats();
}

void SimulationVisualization::resetView()
{
	vtkRenderer *renderer = this->renArea->get_vtk_renderer();
	Configuration *conf = Configuration::getInstance();
	double envSize = conf->getEnvironmentScale();
	renderer->RemoveAllViewProps();
	renderer->Clear();
	renderer->GetActiveCamera()->SetPosition(0.0, 0.0, -3.0 * envSize);
	renderer->GetActiveCamera()->SetViewUp(0.0, 1.0, 0.0);
	renderer->GetActiveCamera()->SetFocalPoint(0.0, 0.0, 0.0);
	renderer->GetActiveCamera()->SetClippingRange(1.0, 4.0 * envSize);

	this->resetOnScreenStats();
	this->addOnScreenStats();
}

void SimulationVisualization::addVisibleItem(vtkProp *prop)
{
	this->renArea->get_vtk_renderer()->AddActor(prop);
}

void SimulationVisualization::addOnScreenStat(OnScreenStatistic *s)
{
	this->onScreenStats.push_back(s);
	this->addVisibleItem(s->getText());
	s->getText()->SetDisplayPosition(10, this->nextOnScreenStatHeight);
	this->nextOnScreenStatHeight += 15;
}

void SimulationVisualization::update(double interval)
{
	std::list<OnScreenStatistic *>::iterator it;
	for (it = this->onScreenStats.begin(); it != this->onScreenStats.end(); it++)
		(*it)->update(interval);
	this->renArea->queue_draw();
}

void SimulationVisualization::resetOnScreenStats()
{
	this->onScreenStats.clear();
	this->nextOnScreenStatHeight = BASE_ON_SCREEN_STAT_HEIGHT;
}

void SimulationVisualization::addOnScreenStats()
{
	Configuration *conf = Configuration::getInstance();
	if (conf->shouldShowFPS())
	{
		this->addOnScreenStat(new OnScreenFPS());
	}
	if (conf->shouldShowCPS())
	{
		this->addOnScreenStat(new OnScreenCPS());
	}
}