#include "vector.hh"
#include "mtrand.h"
#include <cmath>

Vector::Vector() : x(0), y(0), z(0)
{
}

Vector::Vector(double x, double y, double z) : x(x), y(y), z(z)
{
}

Vector::Vector(double v[3]) : x(v[0]), y(v[1]), z(v[2])
{
}

Vector Vector::operator+(Vector v)
{
	return Vector(x + v.x, y + v.y, z + v.z);
}

Vector Vector::operator-(Vector v)
{
	return Vector(x - v.x, y - v.y, z - v.z);
}

Vector Vector::operator*(double scalar)
{
	return Vector(scalar * x, scalar * y, scalar * z);
}

bool Vector::operator==(Vector v)
{
	return (*this - v).length() < 0.001;
}

bool Vector::operator!=(Vector v)
{
	return !(*this == v);
}

Vector Vector::randomDirection()
{
	MTRand drand;
	Vector direction(-1.0 + 2.0 * drand(), -1.0 + 2.0 * drand(), -1.0 + 2.0 * drand());
	direction = direction * (1.0 / direction.length());
	return direction;
}

Vector Vector::direction()
{
	return Vector(x / length(), y / length(), z / length());
}

double Vector::dot(Vector v)
{
	return x * v.x + y * v.y + z * v.z;
}

Vector Vector::multiply(Vector v)
{
	return Vector(y * v.z - v.y * z, v.x * z - x * v.z, x * v.y - v.x * y);
}

Vector Vector::reflect(Vector normal)
{
	Vector normalized = normal * (1.0 / normal.length());
	return *this - normalized * (this->dot(normalized) * 2.0);
}

double Vector::length()
{
	return sqrt(this->dot(*this));
}

std::string Vector::toString()
{
	char buf[50];
	sprintf(buf, "(%.3f, %.3f, %.3f)", x, y, z);
	return std::string(buf);
}
