#include <cmath>
#include "vectorTest.hh"

CPPUNIT_TEST_SUITE_REGISTRATION(VectorTest);

void VectorTest::setUp()
{
	v1 = Vector(1.0, 2.0, 3.0);
	v2 = Vector(3.0, 2.0, 1.0);
	v3 = Vector(1.0, 0.0, 0.0);
	v4 = Vector(0.0, 1.0, 0.0);
	v5 = Vector(0.0, 0.0, 1.0);
}

void VectorTest::tearDown()
{
}

void VectorTest::testSum()
{
	Vector res = Vector(4.0, 4.0, 4.0);
	CPPUNIT_ASSERT(res == v1 + v2);
}

void VectorTest::testSubtract()
{
	Vector res = Vector(-2.0, 0.0, 2.0);
	CPPUNIT_ASSERT(res == v1 - v2);
}

void VectorTest::testScalarMult()
{
	Vector res = Vector(4.0, 8.0, 12.0);
	CPPUNIT_ASSERT(res == v1 * 4.0);
}

void VectorTest::testDot()
{
	CPPUNIT_ASSERT(0.0 == v4.dot(v5));
	CPPUNIT_ASSERT(10.0 == v1.dot(v2));
}

void VectorTest::testVectorMult()
{
	Vector res = Vector(0.0, 0.0, 0.0);
	CPPUNIT_ASSERT(v5 == v3.multiply(v4));
	CPPUNIT_ASSERT(res == v1.multiply(v1));
}

void VectorTest::testLength()
{
	Vector v = Vector(1.0, 2.0, 2.0);
	CPPUNIT_ASSERT(3.0 == v.length());
}

void VectorTest::testCompare()
{
	Vector v1 = Vector(1.0, 1.0, 1.1);
	Vector v2 = Vector(1.0, 1.0, 1.0);
	Vector v3 = Vector(1.0, 1.0, 1.1);
	CPPUNIT_ASSERT(true == (v1 == v1));
	CPPUNIT_ASSERT(false == (v1 == v2));
	CPPUNIT_ASSERT(true == (v1 == v3));
}

void VectorTest::testReflect()
{
	Vector result1 = Vector(-1.0, 2.0, 3.0);
	Vector result2 = Vector(1.0, -2.0, 3.0);
	Vector result3 = Vector(1.0, 2.0, -3.0);
	CPPUNIT_ASSERT(result1 == v1.reflect(v3));
	CPPUNIT_ASSERT(result2 == v1.reflect(v4));
	CPPUNIT_ASSERT(result3 == v1.reflect(v5));
}

void VectorTest::testRandomDirection()
{
	int i;
	for (i = 0; i < 1000; i++)
		CPPUNIT_ASSERT(fabs(1.0 - Vector::randomDirection().length()) < 0.001);
}

void VectorTest::testDirection()
{
	CPPUNIT_ASSERT(v3.direction() == v3);
	CPPUNIT_ASSERT(v4.direction() == v4);
	CPPUNIT_ASSERT(v5.direction() == v5);
	CPPUNIT_ASSERT(Vector(5.0, 0.0, 0.0).direction() == v3);
	CPPUNIT_ASSERT(Vector(1.0, 1.0, 1.0).direction() == Vector(5.2, 5.2, 5.2).direction());
	CPPUNIT_ASSERT(Vector(3.0, 2.0, 5.0).direction() == Vector(9.0, 6.0, 15.0).direction());
}